# This script calculates the CRPS deterioration over the forecast lead time 

#-------------------------------------

#Specify here the GloFAS station ID of interest e.g. G5378

station_id="G5249"

#-------------------------------------

#Load data:
data=readMat(paste(station_id,"_glofasdata.mat",sep=''))
data=unlist(data) 
dataarray=array(data,dim=c(2830,51,31)) 
startdate=as.numeric(as.Date(c("2008-04-01")))

#Extract the water balance data:
wbdata=dataarray[,1,1]

#Reshape the water balance list into a 2D array, columns of 30:
wb=wbdata[2:31]
for(i in 2:2799){wb=rbind(wb,wbdata[(i+1):(i+30)])}

#Load observed data
observed=read.csv(paste(station_id,"_observed.csv",sep=''))
preobs=observed[,1]
obs=preobs[2:31]
for(i in 2:2799){obs=rbind(obs,preobs[(i+1):(i+30)])}

#Create an empty array for the CRPS:
#Loop over lead times to compute the CRPS at each:
CRPSfinalwb = rep(0,30)
for(i in 1:30){crps=crpsDecomposition(wb[,i],dataarray[1:2799,,i+1])$CRPS
CRPSfinalwb[i]=crps}

print(CRPSfinalwb)

CRPSfinalobs = rep(0,30)
for(i in 1:30){crps=crpsDecomposition(obs[,i],dataarray[1:2799,,i+1])$CRPS
CRPSfinalobs[i]=crps}

print(CRPSfinalobs)

#Plot deterioration of CRPS with lead time:
axis_range=range(0, CRPSfinalobs)
plot(CRPSfinalwb,ann=FALSE,ylim=axis_range)
lines(1:30,CRPSfinalobs,type="p",pch=16,bg="black")
title(main="CRPS Deterioration with Forecast Lead Time",xlab="Lead Time (Days)",ylab="CRPS")
legend("bottomright", c("Water Balance", "Observed Discharge"), col = c("black", "black"),
       text.col = "black", pch = c(1, 16))



