# This script calculates the NSE deterioration over the forecast lead time 

#-------------------------------------

#Specify here the GloFAS station ID of interest e.g. G5378

station_id="G5249"

#-------------------------------------

#Load data:
data=readMat(paste(station_id,"_glofasdata.mat",sep=''))
data=unlist(data) 
dataarray=array(data,dim=c(2830,51,31)) 
startdate=as.numeric(as.Date(c("2008-04-01")))

#Extract the water balance data only:
wbdata=dataarray[,1,1]

#Reshape the water balance list into a 2D array, columns of 30:
wb=wbdata[2:31]
for(i in 2:2799){wb=rbind(wb,wbdata[(i+1):(i+30)])}

#Create an array for the forecast data:
fcst=matrix(nrow=2830,ncol=31)

#Loop over days and lead times and add data to array
for(i in 1:2830){for(j in 1:31){fcst[i,j]=mean(dataarray[i,,j])}}

#Remove the water balance from the array
#Remove the last 30 forecast dates as no WB to compare to:
fcst=fcst[1:2799,2:31]

#Compute Nash-Sutcliffe Efficiency 
nse = NSE(fcst,wb,na.rm=TRUE)
print(nse)

#png(paste(station_id,"_NSE_deterioration.png",sep=''))

#Plot the NSE deterioration with lead time:
axis_range=range(nse, 1)
plot(nse,ann=FALSE,ylim=axis_range)
lines(0:31,rep(0,32))
title(main="Nash Sutcliffe Efficiency: Deterioration with Forecast Lead Time",xlab="Lead Time (Days)",ylab="NSE")

#dev.off()


