# This script plots a ROC Curve for a specified lead time and event threshold 

#-------------------------------------

#Specify here the GloFAS station ID of interest e.g. G5378

station_id="G5249"

#Specify here the forecast lead time for which to plot the ROC curve

lt=10

#Specify here whether you want to produce the curve using the model water balance ("wb)
#or observations ("obs")

wbobs="obs"

#Specify the percentile to classify an event (if the water balance exceeds this percentile, event = TRUE)
#90th percentile = 10, 80th percentile = 9, 70th percentile =8, 60th percentile =7, etc...

userpctl=9



#-------------------------------------

#Load data:
data=readMat(paste(station_id,"_glofasdata.mat",sep=''))
data=unlist(data) 
dataarray=array(data,dim=c(2830,51,31)) 
startdate=as.numeric(as.Date(c("2008-04-01")))

#Extract the water balance data:
wbdata=dataarray[,1,1]

#print(wbdata)

#Reshape the water balance list into a 2D array, columns of 30:
wb=wbdata[2:31]
for(i in 2:2799){wb=rbind(wb,wbdata[(i+1):(i+30)])}

#Compute the 90th percentile of climatology:
pctlwb=quantile(wbdata,probs=seq(0,1,by=0.1),na.rm=TRUE)

#print(pctl)

threswb=pctlwb[userpctl]

#print(thres)

#Create an array for the binary event/non-event from WB
binwb=matrix(nrow=2799,ncol=30,data=0)

#Loop over WB and if it exceeded the threshold, set to 1
for(i in 1:2799){
  for(j in 1:30){
    if(is.na(wb[i,j])==TRUE){
      binwb[i,j]=NaN
    }
    else if(wb[i,j]>=threswb){
      binwb[i,j]=1
    }
  }}

#print(binwb)

#Create an array for the percentage of forecast members above the threshold
pfcstwb=matrix(nrow=2799,ncol=30)

#Calculate % of forecast members exceeding the threshold
for(i in 1:2799){for(j in 1:30){pfcstwb[i,j]=sum(dataarray[i,,j]>=threswb)/51}}

aucwb=roc.area(binwb[,lt],pfcstwb[,lt])
if(wbobs=="wb"){
  print("Area under curve:")
  print(aucwb[1])
  print("Number of Events:")
  print(aucwb[3])
  print("Number of Non-Events:")
  print(aucwb[4])
}

savepctl=(userpctl-1)*10

#Load observed data
observed=read.csv(paste(station_id,"_observed.csv",sep=''))
preobs=observed[,1]
obs=preobs[2:31]
for(i in 2:2799){obs=rbind(obs,preobs[(i+1):(i+30)])}

#Compute the 90th percentile of climatology:
pctlobs=quantile(obs,probs=seq(0,1,by=0.1),na.rm=TRUE)
thresobs=pctlobs[userpctl]
#Create an array for the binary event/non-event from WB
binobs=matrix(nrow=2799,ncol=30,data=0)

#Loop over obs and if it exceeded the threshold, set to 1
for(i in 1:2799){
  for(j in 1:30){
    if(is.na(obs[i,j])==TRUE){
      binobs[i,j]=NaN
    }
    else if(obs[i,j]>=thresobs){
      binobs[i,j]=1
    }
  }}

#Create an array for the percentage of forecast members above the threshold
pfcstobs=matrix(nrow=2799,ncol=30)

#Calculate % of forecast members exceeding the threshold
for(i in 1:2799){for(j in 1:30){pfcstobs[i,j]=sum(dataarray[i,,j]>=thresobs)/51}}

aucobs=roc.area(binobs[,lt],pfcstobs[,lt])
if(wbobs=="obs"){
  print("Area under curve:")
  print(aucobs[1])
  print("Number of Events:")
  print(aucobs[3])
  print("Number of Non-Events:")
  print(aucobs[4])
}

if(wbobs=="wb"){roc.plot(binwb[,lt],pfcstwb[,lt],thresholds=seq(0,1,0.1),show.thres=FALSE)}
if(wbobs=="obs"){roc.plot(binobs[,lt],pfcstobs[,lt],thresholds=seq(0,1,0.1),show.thres=FALSE)}

