# This script finds the earliest lead time at which a given flood event was forecast

#-------------------------------------

#Specify here the GloFAS station ID of interest e.g. G5378

station_id="G5249"

#Specify the date of the flood event you are interested in e.g. "2012-02-10"
#Format: "yyyy-mm-dd"

flooddate="2012-02-10"

#Specify the percentage of ensemble members that should forecast this event 
#e.g. 60% or more of the ensemble members should indicate a flood event will occur

perc=60

#-------------------------------------

#Load data:
data=readMat(paste(station_id,"_glofasdata.mat",sep=''))
data=unlist(data) 
dataarray=array(data,dim=c(2830,51,31)) 
startdate=as.numeric(as.Date(c("2008-04-01")))
forecastdate=as.numeric(as.Date(c(flooddate)))
                        
row=(forecastdate-startdate)

#print(forecastdate)

wb=dataarray[row,1,1]

#print(dataarray[row,,1])

print(wb)

thres=wb*0.9
days=0

for(i in 1:30){
  fperc=sum(dataarray[row-i,,i+1]>=thres)/51*100
  if(fperc>=perc){
    days=i
    fpercf=fperc}}


cat("This event was predicted",days,"days in advance, with",fpercf,"% certainty.")

#png(paste(station_id,"_event_forecast_",days,"dayleadtime.png",sep=''))

plot(days,wb,type="p",col="red")
#boxplot(dataarray[row-days,,])

axis_range=range(0,dataarray[row-days,,])
plot(1:31,dataarray[row-days,1,],type="n",ann=FALSE,ylim=axis_range)
for (i in 1:51){lines(1:31,dataarray[row-days,i,],type="l",ann=FALSE,col="dark grey")}

wbvec=rep(0,31)
for(i in 1:31){wbvec[i]=dataarray[(row-days)+(i-1),1,1]}
lines(1:31,wbvec,col="red",type="l")
lines(days+1,wb,col="red",type="p",pch=15,bg="red")

legend("topleft", c("Forecast", "Water Balance", "Event WB"), col = c("dark grey", "red", "red"),
       text.col = "black", lty = c(1, 1, NA), pch = c(NA, NA, 16),
       merge = TRUE)
title(main=paste("Forecast of event from",days,"days ahead"),xlab="Lead Time (Days)",ylab="River Discharge")

#dev.off()




